#include <talk.h>

Talk *talk;

/* start and stop functions */
gboolean start_plugin(gint argc,gchar *argv[]) {

  Client *c = (Client*)argv[argc+1];

  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
       g_hash_table_lookup(c->plugin_buttons,"talk"))) == TRUE) {
    talk = talk_new();
    talk->c = c;
    talk_app_open_cb(NULL,talk);
    return TRUE;
  }

  current_error_message_set(_("The Talk plugin has already been loaded"));
  return FALSE;

}

gboolean stop_plugin(gint argc,gchar *argv[]) {

  Client *c = (Client*)argv[argc+1];

  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
       g_hash_table_lookup(c->plugin_buttons,"talk"))) == FALSE) {
    gtk_widget_destroy(talk->talk_app);
    return TRUE;
  }
  if(argc >= 1) {
    if(g_strcasecmp(argv[0],"free")==0) {
      talk_destroy(talk);
    }
  }

  current_error_message_set(_("The Talk plugin has already been unloaded"));
  return FALSE; 

}



/* network functions */
gboolean ta_send_message_through_server(gint argc,gchar *argv[]) {

  gboolean rv = FALSE;

  if(argc == 3) {
    User *to_user = user_get(((Server*)argv[argc+1])->users,argv[1]);

    if(to_user == NULL) {
      gchar *str = g_strdup_printf(_("Can't send a message to %s, that user is not logged on\n"),argv[1]);
      current_error_message_set(str);
      g_free(str);
      rv = FALSE;
    } else {
      gchar *args[] = {
	argv[0],
	argv[1],
	argv[2],
	NULL
      };
      rv = call_remote_func(to_user->c,"talk","ta_recv_message",args);
    }
  } else {
    gchar *str = 
      g_strdup_printf(_("%s: wrong number of arguments got %d need %d"),
	              __FUNCTION__,argc,3);
    current_error_message_set(str);
    g_free(str);    
  }

  return rv;

}

gboolean ta_recv_message(gint argc,gchar *argv[]) {

  gboolean rv = FALSE;
  
  if(argc == 3) {
    talk_recv_message(talk,argv[0],argv[2]);
    rv = TRUE;
  } else {
    gchar *str = 
      g_strdup_printf(_("%s: wrong number of arguments got %d need %d"),
	              __FUNCTION__,argc,3);
    current_error_message_set(str);
    g_free(str);    
  }

  return rv;

}



/* local functions, opening the app, user interface stuff... */

Talk *talk_new(void) {

  Talk *new = (Talk*)g_malloc0(sizeof(Talk));

  /* FIXME: replace with session management */
  new->talk_fg_color = "Black";
  new->talk_bg_color = "White";
  new->talk_bg_pixmap = "" ;
  new->talk_font = "fixed";

  return new;
  
}

void talk_destroy(Talk *t) {

  if(t == NULL) return;
  g_free(t);
  t = NULL;

}

gint talk_user_name_compare_func(gconstpointer a,gconstpointer b) {

  if(!strcmp((gchar*)a,(gchar*)b)) return 0;
  return 1;
  
}

void talk_select_user_cb(GtkCList *clist,gint row,gint column,
                         GdkEventButton *event, Talk *t) {
  gchar *name = NULL;
  
  gtk_clist_get_text(clist,row,NAME_COLUMN,&name);
  t->to_users_list = g_list_append(t->to_users_list,g_strdup(name));
  
}

void talk_unselect_user_cb(GtkCList *clist,gint row,gint column,
                           GdkEventButton *event, Talk *t) {
  gchar *name = NULL;
  GList *tmp = NULL;
  
  gtk_clist_get_text(clist,row,NAME_COLUMN,&name);
  tmp = g_list_find_custom(t->to_users_list,name,talk_user_name_compare_func);

  if(tmp != NULL) {
    t->to_users_list = g_list_remove_link(t->to_users_list,tmp);
    g_free(tmp->data);
    g_list_free_1(tmp);
  }
  
}

void talk_send_message_cb(GtkWidget *widget,gpointer data) {

  Talk *t = (Talk*)data;
  Client *c = t->c;
  gchar *lib = "talk";
  gchar *func = "ta_send_message_through_server";
  gchar *message = gtk_editable_get_chars(GTK_EDITABLE(t->talk_entry),0,-1);

  if(c->server != NULL) {
    if((strlen(lib) > 0) && (strlen(func) > 0) && (strlen(message) > 0)) {
      GList *user = t->to_users_list;
      
      while(user != NULL) {
	gchar *argv[] = {
          g_strdup(c->my_username),
	  g_strdup((gchar*)user->data),
	  g_strdup(message),
	  NULL
        };

	call_remote_func(c->server,lib,func,argv);

	g_free(argv[0]);
        g_free(argv[1]);
        g_free(argv[2]);

	user = user->next;
      }
    }
  }

  gtk_entry_set_text(GTK_ENTRY(t->talk_entry),(const gchar*)"");
  g_free(message);

}

gboolean talk_recv_message(Talk *t,gchar *from,gchar *message) {

  GdkColor fg_color,bg_color;
  GdkFont *font = NULL;
  gchar *str = NULL;

  gdk_color_parse(t->talk_fg_color,&fg_color);
  gdk_color_parse(t->talk_bg_color,&bg_color);
  font = gdk_font_load(t->talk_font);
  
  
  gtk_text_insert(GTK_TEXT(t->talk_text),font,&fg_color,&bg_color,
                  str = g_strconcat(from,": ",message,"\n",NULL),-1);

  gdk_font_unref(font);
  g_free(str);

  return TRUE;
}

void talk_click_column_cb(GtkCList *clist, gint column, gpointer data) {

  switch(column) {
    case 1:
      talk_add_user_cb(GTK_WIDGET(clist),data);
      break;
    default:
      break;
  }
  
}

void talk_add_user_cb(GtkWidget *widget,gpointer data) {

  Talk *t = (Talk*)data;
  GtkWidget *dialog,*entry,*label;
  gint reply;
  
  /* I decide not to save this dialog, to just generate it
   * every time someone adds a user.
   */
  dialog = gnome_dialog_new(_("Add User"),
                            GNOME_STOCK_BUTTON_OK,
			    GNOME_STOCK_BUTTON_CANCEL,
			    NULL);

  label = gtk_label_new(_("Please enter the user name of the person\nyou want to add to your list..."));
  gtk_label_set_justify(GTK_LABEL(label),GTK_JUSTIFY_LEFT);
  entry = gtk_entry_new();

  gnome_dialog_set_close(GNOME_DIALOG(dialog),TRUE);
  gnome_dialog_close_hides(GNOME_DIALOG(dialog),TRUE);
  gnome_dialog_editable_enters(GNOME_DIALOG(dialog),GTK_EDITABLE(entry));
  gnome_dialog_set_default(GNOME_DIALOG(dialog),GNOME_OK);

  gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox),label,
                     TRUE,TRUE,GNOME_PAD);
  gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox),entry,
                     TRUE,TRUE,GNOME_PAD);

  gtk_widget_show_all(dialog);

  reply = gnome_dialog_run(GNOME_DIALOG(dialog));

  if(reply == GNOME_OK) {
    gchar *name = gtk_editable_get_chars(GTK_EDITABLE(entry),0,-1);
    gchar *args[] = {
      "",
      name
    };
    
    gtk_clist_prepend(GTK_CLIST(t->talk_clist),args);
    
    g_free(name);
  }

  gtk_widget_destroy(dialog);
  
}

void talk_stop_cb(GtkWidget *widget,gpointer data) {

  Talk *t = (Talk*)data;
  gchar *args[] = { "free", NULL, (gchar*)t->c };
  
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(g_hash_table_lookup(t->c->plugin_buttons,"talk")),FALSE);
  stop_plugin(1,(gchar**)args);

}

void talk_app_open_cb(GtkWidget *widget,gpointer data) {

  Talk *t = (Talk*)data;
  gchar *str;
  gchar *clist_titles[] = {
    "",
    _("add name")
  };
  
  t->talk_app =
    gnome_app_new(PACKAGE,(str=g_strdup_printf(_("%s Talk"),PACKAGE)));  
  
  gtk_signal_connect(GTK_OBJECT(t->talk_app),"destroy",
		     GTK_SIGNAL_FUNC(talk_stop_cb),t);
  
  t->talk_table = gtk_table_new(3,3,FALSE);
  
  t->talk_entry = gtk_entry_new();
  t->talk_text = gtk_text_new(NULL,NULL);
  t->talk_clist = gtk_clist_new_with_titles(N_COLUMNS,clist_titles);

  gtk_clist_set_reorderable(GTK_CLIST(t->talk_clist),TRUE);
  gtk_clist_set_selection_mode(GTK_CLIST(t->talk_clist),
                               GTK_SELECTION_MULTIPLE);
  gtk_signal_connect(GTK_OBJECT(t->talk_clist),"click-column",
                     GTK_SIGNAL_FUNC(talk_click_column_cb),t);  
  gtk_signal_connect(GTK_OBJECT(t->talk_clist),"select-row",
                     GTK_SIGNAL_FUNC(talk_select_user_cb),t);  
  gtk_signal_connect(GTK_OBJECT(t->talk_clist),"unselect-row",
                     GTK_SIGNAL_FUNC(talk_unselect_user_cb),t);  
  
  gtk_signal_connect(GTK_OBJECT(t->talk_entry),"activate",
		     GTK_SIGNAL_FUNC(talk_send_message_cb),t);    
  
  gtk_table_attach_defaults(GTK_TABLE(t->talk_table),
			    t->talk_text,0,1,0,1);
  gtk_table_attach_defaults(GTK_TABLE(t->talk_table),
			    t->talk_entry,0,1,1,2);
  gtk_table_attach_defaults(GTK_TABLE(t->talk_table),
			    t->talk_clist,1,2,0,2);
  
  gnome_app_set_contents(GNOME_APP(t->talk_app),t->talk_table);
  
  gtk_widget_show_all(t->talk_app);
  
  g_free(str);
  
}


