#include <client.h>

static GnomeUIInfo client_file_menu[] = {
  GNOMEUIINFO_MENU_EXIT_ITEM(client_app_destroy_cb,NULL),
  GNOMEUIINFO_END
};
static GnomeUIInfo client_edit_menu[] = {
  GNOMEUIINFO_MENU_PROPERTIES_ITEM(prop_dialog_cb,NULL),
  GNOMEUIINFO_END
};
static GnomeUIInfo client_help_menu[] = {
  GNOMEUIINFO_MENU_ABOUT_ITEM(about_dialog_cb,NULL),
  GNOMEUIINFO_END
};
static GnomeUIInfo client_menu[] = {
  GNOMEUIINFO_MENU_FILE_TREE(client_file_menu),
  GNOMEUIINFO_MENU_EDIT_TREE(client_edit_menu),
  GNOMEUIINFO_MENU_HELP_TREE(client_help_menu),
  GNOMEUIINFO_END
};

Client *client_new(void) {

  Client *new = (Client*)g_malloc0(sizeof(Client));

  new->loaded_modules = module_hash_new();
  new->users = g_hash_table_new(user_hash_func,user_compare_func);
  new->plugin_buttons = g_hash_table_new(g_str_hash,g_str_equal);

  private_plugin_dir = NULL;
  client_or_server = (ClientOrServer*)new;

  new->my_initial_state = "USER_ONLINE";
  
  return new;

}

void client_destroy_cb(GtkWidget *widget,gpointer data) {
  
  client_prop_save_geometry_cb(NULL,NULL,data);
  client_destroy((Client*)data);

}

void client_app_destroy_cb(GtkWidget *widget,gpointer data) {

  Client *c = (Client*)data;

  if(c->app != NULL) {
    gtk_widget_destroy(c->app);
    c->app = NULL;
  }
  
}

void client_destroy(Client* c) {

  g_print("client destroy!\n");
  if(c->loaded_modules != NULL) {
    module_hash_destroy(c->loaded_modules);
    c->loaded_modules = NULL;
  }
  if(c->users != NULL) {
    g_hash_table_foreach_remove(c->users,user_remove_func,NULL);
    g_hash_table_destroy(c->users);
  }
  if(c->plugin_buttons != NULL) {
    /* FIXME: destroy all the toggle buttons... */
    g_hash_table_destroy(c->plugin_buttons);
  }
  if(c->server != NULL) {
    connection_destroy(c->server);
    c->server = NULL;
  }
  
  g_free(c);
  c = NULL;

  gtk_main_quit();
  
}

void client_init(Client *c,gchar *geometry) {
  
  gchar *str;
  gint i;
  
  g_assert(c != NULL);
  
  /* GUI */
  c->app = 
    gnome_app_new(PACKAGE,(str=g_strdup_printf(_("%s %s"),PACKAGE,VERSION)));  

  gnome_app_create_menus_with_data(GNOME_APP(c->app),client_menu,c);
  gtk_window_set_policy(GTK_WINDOW(c->app),TRUE,TRUE,TRUE);
  
  c->table = gtk_table_new(2,5,FALSE);

  c->plugin_scroll = gtk_scrolled_window_new(NULL,NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(c->plugin_scroll),
                                 GTK_POLICY_NEVER,GTK_POLICY_AUTOMATIC);

  c->plugin_table = gtk_table_new(2,2,FALSE);
  gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(c->plugin_scroll),
                                        c->plugin_table);
  
  c->state_menu = gtk_menu_new();
  for(i=1;i<N_USER_STATES;i++) {
    /* i=1, and not 0, because 0 is offline, if you want to do offline,
     * just use the disconnect button...
     */
    c->state_item[i] = gtk_menu_item_new_with_label(user_state_string_nice[i]);
    gtk_signal_connect(GTK_OBJECT(c->state_item[i]),"activate",
	               GTK_SIGNAL_FUNC(client_state_option_changed_cb),c);
    gtk_menu_append(GTK_MENU(c->state_menu),c->state_item[i]);
  }
  c->state_option = gtk_option_menu_new();
  gtk_option_menu_set_menu(GTK_OPTION_MENU(c->state_option),c->state_menu);
  gtk_widget_set_sensitive(c->state_option,FALSE);
    
  c->connect_toggle = gtk_toggle_button_new_with_label(_("connect to server"));

  gtk_table_attach(GTK_TABLE(c->table),c->plugin_scroll,0,1,0,1,
                   GTK_EXPAND | GTK_SHRINK | GTK_FILL,
		   GTK_EXPAND | GTK_SHRINK | GTK_FILL,
		   0,0);
  gtk_table_attach(GTK_TABLE(c->table),c->state_option,0,1,2,3,
                   GTK_EXPAND | GTK_SHRINK | GTK_FILL,
		   GTK_FILL,
                   0,0);
  gtk_table_attach(GTK_TABLE(c->table),c->connect_toggle,0,1,3,4,
                   GTK_EXPAND | GTK_SHRINK | GTK_FILL,
		   GTK_FILL,
                   0,0);


  /* POPT */
  if(geometry != NULL) {
    gint x,y,width,height;
    if(gnome_parse_geometry(geometry,&x,&y,&width,&height)) {
      if(x != -1) {
	gtk_widget_set_uposition(c->app,x,y);
      }
      if(width != -1) {
	gtk_window_set_default_size(GTK_WINDOW(c->app),width,height);
      }
      client_prop_save_geometry_cb(NULL,NULL,c);
    } else {
      g_error(_("Could not parse geometry string `%s'"),geometry);
    }
  } else {
    client_prop_load_geometry_cb(NULL,NULL,c);
  }


  /* SIGNALS */
  gtk_signal_connect(GTK_OBJECT(c->app),"destroy",
		     GTK_SIGNAL_FUNC(client_destroy_cb),c);

  gtk_signal_connect(GTK_OBJECT(c->connect_toggle),"toggled",
                     client_connect_toggle_cb,c);

  gnome_app_set_contents(GNOME_APP(c->app),c->table);


  /* SESSION MANAGEMENT */
  c->session_client = gnome_master_client();
  gtk_signal_connect(GTK_OBJECT(c->session_client),"save_yourself",
                     GTK_SIGNAL_FUNC(client_prop_save_cb),c);
  client_prop_load_cb(NULL,NULL,c); /* you can find the session management code 
                                       at the bottom of this file... */
 
 
  gtk_widget_show_all(c->app);
 
  /* SCAN FOR PLUGINS, this needs to be done after gtk_widget_show_all(),
   * usually client_prop_load_cb() calls it, but that function got called
   * before gtk_widget_show_all()
   */
  client_scan_plugins_cb(c->plugin_table,c);
  
  g_free(str);
  
}

/* a local function, not in the header file */
gboolean toggle_button_remove_func(gpointer key,gpointer value,gpointer data) {

  gtk_widget_destroy(GTK_WIDGET(value));
  return TRUE;
  
}

void client_scan_plugins_cb(GtkWidget *widget,gpointer data) {

  Client *c = (Client*)data;
  GList *plugins = get_plugins();
  gint i = 0;
  
  g_hash_table_foreach_remove(c->plugin_buttons,
                              (GHRFunc)toggle_button_remove_func,NULL);
  gtk_widget_destroy(c->plugin_table);

  c->plugin_table = gtk_table_new(2,g_list_length(plugins)+1,FALSE);
  gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(c->plugin_scroll),
                                        c->plugin_table);

  i = 0;
  while(plugins != NULL) {
    GtkWidget *toggle;
    gchar *str = (gchar*)plugins->data, *label;
    
    label = strtok(str,SEPARATOR); /* this is the dir, we don't want that */
    label = strtok(NULL,SEPARATOR); /* this is what we want, the plugin name */
    
    toggle = gtk_toggle_button_new_with_label(label);
    g_hash_table_insert(c->plugin_buttons,label,toggle);
    gtk_table_attach_defaults(GTK_TABLE(c->plugin_table),toggle,0,1,i,i+1);
    gtk_signal_connect(GTK_OBJECT(toggle),"toggled",client_toggle_plugin_cb,c);
    gtk_widget_show(toggle);
    
    plugins = g_list_remove(plugins,str);
    i++;
  }
  
  gtk_widget_show(c->plugin_table);

}

void client_toggle_plugin_cb(GtkWidget *widget,gpointer data) {

  Client *c = (Client*)data;
  gchar *str = NULL;
  gchar *args[] = { NULL, (gchar*)c };
  
  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget))) {
    gtk_label_get(GTK_LABEL(GTK_BIN(widget)->child),&str);
    g_print("%s: in %s\n",__FUNCTION__,str);
    if(str != NULL) {
      call_func(c->loaded_modules,str,"start_plugin",0,(const gchar**)args);
    }
  } else {
    gtk_label_get(GTK_LABEL(GTK_BIN(widget)->child),&str);
    g_print("%s: out %s\n",__FUNCTION__,str);
    if(str != NULL) {
      call_func(c->loaded_modules,str,"stop_plugin",0,(const gchar**)args);
    }
  }
}


void client_state_option_changed_cb(GtkWidget *widget,gpointer data) {

  g_print("changed\n");
  
}

void client_connect_toggle_cb(GtkWidget *widget,gpointer data) {

  Client *c = (Client*)data;

  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(c->connect_toggle))) {
    if(client_connect_cb(widget,data)) {
      gtk_label_set_text(GTK_LABEL(GTK_BIN(c->connect_toggle)->child),
  	                 _("disconnect from server"));
      gtk_widget_set_sensitive(c->state_option,TRUE);
    } else {
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(c->connect_toggle),FALSE);
    }
  } else {
    if(client_disconnect_cb(widget,data)) {
      gtk_label_set_text(GTK_LABEL(GTK_BIN(c->connect_toggle)->child),
  	                 _("connect to server"));
      gtk_widget_set_sensitive(c->state_option,FALSE);
    } else {
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(c->connect_toggle),TRUE);
    }
  }
}

gboolean client_connect_cb(GtkWidget *widget,gpointer data) {

  Client *c = (Client*)data;
  gboolean rv = FALSE;

  if(c->server == NULL) {
    c->server = connection_new();
  }
  if((c->my_server_ip != NULL) && (strlen(c->my_server_ip) > 0)) {
    if(c->my_server_port != 0) {

      if(c->server->ip != NULL) {
	  g_free(c->server->ip);
      }
      c->server->ip = g_strdup(c->my_server_ip);
      c->server->port = c->my_server_port;
      
      g_print("server: %s %d\n",c->server->ip,c->server->port);
      
      if(connection_open(c->server) == FALSE) {
	gchar *str = g_strdup_printf(_("can't connect to %s"),c->my_server_ip);
	current_error_message_set(str);
	current_error_message_show();
        g_free(str);
      }
      
      if((c->my_username != NULL) && (strlen(c->my_username) > 0)) {
	if((c->my_password != NULL) && (strlen(c->my_password) > 0)) {
	  gchar *args[] = { 
	    c->my_username, 
	    c->my_password, 
	    c->my_initial_state,
	    NULL
	  };
	  if(call_remote_func(c->server,"users","us_user_login",args)) {
	    rv = TRUE;
	  }
	} else { /* no password */
	  gchar *str = g_strdup_printf(_("please enter your password in the properties window\nbefore connecting to the server\n\nEdit->Properties"));
	  current_error_message_set(str);
	  current_error_message_show();
	  g_free(str);
	  client_disconnect_cb(widget,data);
	}
      } else { /* no username */
        gchar *str = g_strdup_printf(_("please enter your password in the properties window\nbefore connecting to the server\n\nEdit->Properties"));
        current_error_message_set(str);
        current_error_message_show();
        g_free(str);
	client_disconnect_cb(widget,data);
      }
    } else { /* no port */
      gchar *str = g_strdup_printf(_("please enter the port of your Quest server\nin the properties windowbefore trying to connect\n\nEdit->Properties"));
      current_error_message_set(str);
      current_error_message_show();
      g_free(str);
    }
  } else { /* no ip */
    gchar *str = g_strdup_printf(_("please enter the ip address of your Quest server\nin the properties window before trying to connect\n\nEdit->Properties"));
    current_error_message_set(str);
    current_error_message_show();
    g_free(str);
  }

  return rv;

}

gboolean client_disconnect_cb(GtkWidget *widget,gpointer data) {

  Client *c = (Client*)data;
  gboolean rv = FALSE;

  if(c->server != NULL) {
    gchar *args[] = { 
      c->my_username, 
      c->my_initial_state,
      NULL
    };
    call_remote_func(c->server,"users","us_user_logout",args);
    connection_close(c->server);
    rv = TRUE;
  }
 
  return rv;

}

void about_dialog_cb(GtkWidget *widget,gpointer data) {

  Client *c = (Client*)data;

  const gchar *authors[] = {
    "Willem Robert van Hage <wrvh@xs4all.nl>",
    NULL
  };
  
  c->about_href = gnome_href_new("http://quest.sourceforge.net",
	                         _("The Quest homepage"));

  c->about_dialog = 
    gnome_about_new("Quest",VERSION,
	            "(C) 2000 Willem Robert van Hage",
	      	    authors,
	      	    _("RPG / Chat application / File Sharing for Gnome"),
		    NULL/* FIXME: add logo */);
  
  gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(c->about_dialog)->vbox),
	             c->about_href,FALSE,FALSE,0);

  gtk_signal_connect(GTK_OBJECT(c->about_dialog),"destroy",
                     GTK_SIGNAL_FUNC(gtk_widget_destroyed),
		     &(c->about_dialog));

  gnome_dialog_set_parent(GNOME_DIALOG(c->about_dialog),GTK_WINDOW(c->app));
  gtk_widget_show_all(c->about_dialog);

}

void prop_dialog_cb(GtkWidget *widget,gpointer data) {

  Client *c = (Client*)data;

  if(c->prop_dialog == NULL) {

    c->prop_dialog = gnome_property_box_new();
    gnome_dialog_close_hides(GNOME_DIALOG(c->prop_dialog),TRUE);

    gtk_signal_connect(GTK_OBJECT(c->prop_dialog),"apply",
                       GTK_SIGNAL_FUNC(client_prop_save_cb),c);

    /* FIXME: fix the help button! */
    
    /* WHO AM I? username and password */
    c->prop_whoami_table = gtk_table_new(2,3,FALSE);
    c->prop_whoami_label = gtk_label_new(_("Who am I?"));
    c->prop_username_label = gtk_label_new(_("username"));
    c->prop_username_entry = gtk_entry_new();
    c->prop_password_label = gtk_label_new(_("password"));
    c->prop_password_entry = gtk_entry_new();
    gtk_entry_set_visibility(GTK_ENTRY(c->prop_password_entry),FALSE);

    gtk_table_attach_defaults(GTK_TABLE(c->prop_whoami_table),
			      c->prop_username_label,0,1,0,1);
    gtk_table_attach_defaults(GTK_TABLE(c->prop_whoami_table),
			      c->prop_username_entry,1,2,0,1);
    gtk_table_attach_defaults(GTK_TABLE(c->prop_whoami_table),
			      c->prop_password_label,0,1,1,2);
    gtk_table_attach_defaults(GTK_TABLE(c->prop_whoami_table),
			      c->prop_password_entry,1,2,1,2);

    gnome_property_box_append_page(GNOME_PROPERTY_BOX(c->prop_dialog),
				   c->prop_whoami_table,
				   c->prop_whoami_label);

    /* WHERE IS THE SERVER? server ip and port */
    c->prop_address_table = gtk_table_new(2,3,FALSE);
    c->prop_address_label = gtk_label_new("Server address");
    c->prop_ip_label = gtk_label_new(_("ip"));
    c->prop_ip_entry = gtk_entry_new();
    c->prop_port_label = gtk_label_new(_("port"));
    c->prop_port_entry = gtk_entry_new_with_max_length(5);

    gtk_table_attach_defaults(GTK_TABLE(c->prop_address_table),
			      c->prop_ip_label,0,1,0,1);
    gtk_table_attach_defaults(GTK_TABLE(c->prop_address_table),
			      c->prop_ip_entry,1,2,0,1);
    gtk_table_attach_defaults(GTK_TABLE(c->prop_address_table),
			      c->prop_port_label,0,1,1,2);
    gtk_table_attach_defaults(GTK_TABLE(c->prop_address_table),
			      c->prop_port_entry,1,2,1,2);

    gnome_property_box_append_page(GNOME_PROPERTY_BOX(c->prop_dialog),
				   c->prop_address_table,
				   c->prop_address_label);

    /* WHERE ARE MY OWN PLUGINS? private plugin dir */ 
    c->prop_plugin_table = gtk_table_new(3,2,FALSE);
    c->prop_plugin_label = gtk_label_new(_("Plug-ins"));
    c->prop_plugin_dir_label = gtk_label_new(_("personal plugin directory"));
    c->prop_plugin_dir_entry = gtk_entry_new();

    gtk_table_attach_defaults(GTK_TABLE(c->prop_plugin_table),
			      c->prop_plugin_dir_label,0,1,0,1);
    gtk_table_attach_defaults(GTK_TABLE(c->prop_plugin_table),
			      c->prop_plugin_dir_entry,0,1,1,2);
    
    gnome_property_box_append_page(GNOME_PROPERTY_BOX(c->prop_dialog),
				   c->prop_plugin_table,
				   c->prop_plugin_label);


    /* load values into enties etc... */
    client_prop_load_cb(NULL,NULL,c);

    /* attach all the "changed" signals after the widgets may have
     * been refreshed with values from a configuration file.
     */
    gtk_signal_connect(GTK_OBJECT(c->prop_username_entry),"changed",
                       GTK_SIGNAL_FUNC(prop_dialog_changed_cb),c);
    gtk_signal_connect(GTK_OBJECT(c->prop_password_entry),"changed",
                       GTK_SIGNAL_FUNC(prop_dialog_changed_cb),c);

    gtk_signal_connect(GTK_OBJECT(c->prop_ip_entry),"changed",
                       GTK_SIGNAL_FUNC(prop_dialog_changed_cb),c);
    gtk_signal_connect(GTK_OBJECT(c->prop_port_entry),"changed",
                       GTK_SIGNAL_FUNC(prop_dialog_changed_cb),c);
    
    gtk_signal_connect(GTK_OBJECT(c->prop_plugin_dir_entry),"changed",
                       GTK_SIGNAL_FUNC(prop_dialog_changed_cb),c);

    gtk_widget_show_all(c->prop_dialog);

    /* DONE! run the dialog */
    gnome_dialog_run(GNOME_DIALOG(c->prop_dialog));

  } else {
    gtk_widget_show(c->prop_dialog);
  }

}

void client_prop_save_cb(GtkWidget *widget,GdkEvent *event,gpointer data) {
  
  Client *c = (Client*)data;

  gchar *str;

  if(c->prop_dialog == NULL) {
    /* you can't make changes to the properties without a property dialog 
     * so if there's no dialog then nothing has to be saved.
     */ 
    return;
  }

  gnome_config_push_prefix
    (gnome_client_get_global_config_prefix(gnome_master_client()));

  /* WHO AM I? username and password */
  gnome_config_clean_section("client user");
  str = gtk_editable_get_chars(GTK_EDITABLE(c->prop_username_entry),0,-1);
  if(str != NULL) {
    if(c->my_username != NULL) {
      g_free(c->my_username);
    }
    c->my_username = str;
    gnome_config_set_string("client user/username",str);
  }
  str = gtk_editable_get_chars(GTK_EDITABLE(c->prop_password_entry),0,-1);
  if(str != NULL) {
    if(c->my_password != NULL) {
      g_free(c->my_password);
    }
    c->my_password = str;
    gnome_config_set_string("client user/password",str);
  }

  /* WHERE IS THE SERVER? server ip and port */
  gnome_config_clean_section("server address");
  str = gtk_editable_get_chars(GTK_EDITABLE(c->prop_ip_entry),0,-1);
  if(str != NULL) {
    if(c->my_server_ip != NULL) {
      g_free(c->my_server_ip);
    }
    c->my_server_ip = str;
    gnome_config_set_string("server address/ip",str);
  }
  str = gtk_editable_get_chars(GTK_EDITABLE(c->prop_port_entry),0,-1);
  if(str != NULL) {
    c->my_server_port = atoi(str);
    gnome_config_set_string("server address/port",str);
  }

  /* WHERE ARE MY PLUGINS? private plugin dir */
  gnome_config_clean_section("client plugins");
  str = gtk_editable_get_chars(GTK_EDITABLE(c->prop_plugin_dir_entry),0,-1);
  if(str != NULL) {
    if(private_plugin_dir != NULL) {
      g_free(private_plugin_dir);
    }
    private_plugin_dir = str;
    client_scan_plugins_cb(NULL,c);
    gnome_config_set_string("client plugins/dir",str);
  }
  g_print("%s: %s\n",__FUNCTION__,private_plugin_dir);
  
  gnome_config_sync();
  gnome_config_pop_prefix();
  
}

void client_prop_load_cb(GtkWidget *widget,GdkEvent *event,gpointer data) {

  Client *c = (Client*)data;
  gchar *str;

  gnome_config_push_prefix
    (gnome_client_get_config_prefix(c->session_client));

  /* WHO AM I? username and password */
  c->my_username = gnome_config_get_string("client user/username");
  if((c->my_username != NULL) && (c->prop_username_entry != NULL)) {
    gtk_entry_set_text(GTK_ENTRY(c->prop_username_entry),c->my_username);
  }
  c->my_password = gnome_config_get_string("client user/password");
  if((c->my_password != NULL) && (c->prop_password_entry != NULL)) {
    gtk_entry_set_text(GTK_ENTRY(c->prop_password_entry),c->my_password);
  }

  /* WHERE IS THE SERVER? server ip and port */
  c->my_server_ip = gnome_config_get_string("server address/ip");
  if((c->my_server_ip != NULL) && (c->prop_ip_entry != NULL)) {
    gtk_entry_set_text(GTK_ENTRY(c->prop_ip_entry),c->my_server_ip);
  }
  str = gnome_config_get_string("server address/port");
  if(str != NULL) {
    c->my_server_port = atoi(str);
    g_free(str);
  }
  if((c->my_server_port != 0) && (c->prop_port_entry != NULL)) {
    gtk_entry_set_text(GTK_ENTRY(c->prop_port_entry),str);
  }

  /* WHERE ARE MY PLUGINS? private plugin dir */
  private_plugin_dir = gnome_config_get_string("client plugins/dir");
  if((private_plugin_dir != NULL) && (c->prop_plugin_dir_entry != NULL)) {
    gtk_entry_set_text(GTK_ENTRY(c->prop_plugin_dir_entry),private_plugin_dir);
    client_scan_plugins_cb(c->plugin_table,c);
  }

  gnome_config_pop_prefix();

}

void client_prop_save_geometry_cb(GtkWidget *widget,GdkEvent *event,
                                  gpointer data) {

  Client *c = (Client*)data;
  gint width=-1,height=-1;

  width = c->app->allocation.width;
  height = c->app->allocation.height;
 
  gnome_config_push_prefix
    (gnome_client_get_global_config_prefix(gnome_master_client()));

  /* WIDTH HEIGHT */
  gnome_config_clean_section("client geography");
  gnome_config_set_int("client geometry/width",width);
  gnome_config_set_int("client geometry/height",height);

  gnome_config_sync();
  gnome_config_pop_prefix();

}

void client_prop_load_geometry_cb(GtkWidget *widget,GdkEvent *event,
                                  gpointer data) {
    
  Client *c = (Client*)data;
  gint width=-1,height=-1;
  
  gnome_config_push_prefix
    (gnome_client_get_config_prefix(c->session_client));

  /* WIDTH HEIGHT */
  width = gnome_config_get_int("client geometry/width");
  height = gnome_config_get_int("client geometry/height");

  gnome_config_pop_prefix();

  if(width != -1) {
    gtk_window_set_default_size(GTK_WINDOW(c->app),width,height);
  }
}

void prop_dialog_changed_cb(GtkWidget *widget,gpointer data) {

  gnome_property_box_changed(GNOME_PROPERTY_BOX(((Client*)data)->prop_dialog));
  
}

