#include <modules.h>

gchar *private_plugin_dir;

gint gmodule_compare_func(gconstpointer a,gconstpointer b) {

  if(g_strcasecmp(g_module_name((GModule*)a),
		  g_module_name((GModule*)b)) == 0) return TRUE;
  return FALSE;

}

gboolean gmodule_remove_func(gpointer key,gpointer value,gpointer user_data) {

  g_module_close((GModule*)value);
  g_free((gchar*)key);

  /* true in this case means remove the module from the hash,
   * no questions asked...
   */
  return TRUE;

}

GHashTable *module_hash_new(void) {
  
  GHashTable *new =
	  g_hash_table_new(g_str_hash,gmodule_compare_func);

  return new;

}

void module_hash_destroy(GHashTable *hash) {

  if(hash != NULL) {
    gint n;
    n = g_hash_table_foreach_remove(hash,
				    gmodule_remove_func,
				    NULL);
    g_hash_table_destroy(hash);
    hash = NULL;
  }
  
}

gboolean call_func(GHashTable *hash,
		   const gchar *lib,const gchar *func,
		   const gint argc,const gchar *argv[]) {
  
  GModule *mod;
  gboolean (*funcptr)(const gint argc,const gchar *argv[]);
  gboolean rv = FALSE;
  gchar *str = NULL;
  struct stat filestat;
  gint statrv;

  if(!g_module_supported()) {
    g_error("modules are not supported, this program needs module support.\n");
    /* no return, because g_error() calls abort() */
  }

  /* where should I search for the library? */
  
  str = g_module_build_path(QUEST_LIB_DIR,lib);
  statrv = stat(str,&filestat);
  if(statrv != 0) {
    /* if you can't find it in the lib dir then it's a plugin */
    if(private_plugin_dir != NULL) {
      str = g_module_build_path(private_plugin_dir,lib);
      statrv = stat(str,&filestat);
      if(statrv != 0) {
        str = g_module_build_path(QUEST_PLUGIN_DIR,lib);
	statrv = stat(str,&filestat);
	if(statrv != 0) {
	  gchar *error_str = g_strdup_printf(_("module not found:\n%s"),str);
	  current_error_message_set(error_str);
	  g_free(error_str);
	}
      }
    } else {
      str = g_module_build_path(QUEST_PLUGIN_DIR,lib);
      statrv = stat(str,&filestat);
      if(statrv != 0) {
	  gchar *error_str = g_strdup_printf(_("module not found:\n%s"),str);
	  current_error_message_set(error_str);
	  g_free(error_str);
      }
    }
  }

  /* try to load the module */
  if(hash!=NULL) {

    if((mod=g_hash_table_lookup(hash,str))==NULL) {

      mod = g_module_open(str,G_MODULE_BIND_LAZY);
      if(mod==NULL) {
	gchar *error_str = g_strdup_printf(_("could not load module:\n%s"),str);
	current_error_message_set(error_str);
	g_free(error_str);
	return FALSE;
      }

      g_hash_table_insert(hash,g_strdup(str),mod);
    
    }
    
    if(!g_module_symbol(mod,func,(gpointer)&funcptr)) {
      gchar *error_str = g_strdup_printf(_("could not load symbol \"%s\"\n%s"),
	                                 func,g_module_error());
      current_error_message_set(error_str);
      g_free(error_str);
      return FALSE;
    }

    rv = (*funcptr)(argc,argv);

  } else { /* if(hash==NULL) */
   
    mod = g_module_open(str,G_MODULE_BIND_LAZY);
    if(mod==NULL) {
      gchar *error_str = g_strdup_printf(_("could not load module:\n%s"),str);
      current_error_message_set(error_str);
      g_free(error_str);
      return FALSE;
    }

    if(!g_module_symbol(mod,func,(gpointer)&funcptr)) {
      gchar *error_str = g_strdup_printf(_("could not load symbol \"%s\"\n%s"),
	                                 func,g_module_error());
      current_error_message_set(error_str);
      g_free(error_str);
      return FALSE;
    }

    rv = (*funcptr)(argc,argv);

    g_module_close(mod);

  }

  if(str!=NULL) g_free(str);

  return rv;

}
