#ifndef QUEST_MODULES_H
#define QUEST_MODULES_H

#include <gnome.h>
#include <gmodule.h>

#include <sys/stat.h>
#include <unistd.h>

#include <cur_error.h>

extern gchar *private_plugin_dir;

/*
 * PLUGIN FUNCTIONS...
 *
 * the parameters of every plugin are (gint argc, gchar *argv[])
 *
 * argc is the number of strings in argv[] and argv doesn't include
 * the program name, like you're used to in main(int argc,char *argv[]),
 * this is just a system that works similarly, it's not a full analogy.
 *
 * so argv[0] is the first argument of the plugin, not the plugin name
 * or anything like that...
 */


/* these two functions are used by module_hash_new()
 */
gint gmodule_compare_func(gconstpointer a,gconstpointer b);
gboolean gmodule_remove_func(gpointer key,gpointer value,gpointer user_data);

/* these are two functions that you can use to create and destroy
 * a hash table to keep track of your loaded GModule*'s
 */
GHashTable *module_hash_new(void);
void module_hash_destroy(GHashTable *hash);

/* hash is a pointer to a hash table that keeps track of all the loaded
 * modules hashed by their filename including path.
 * you can either pass the hash table where you store your GModule*'s
 * or NULL, in which case the module is unloaded again after use.
 * don't worry about closing a module you will need in the future this
 * way, because the GLIB module system supports reference counting.
 */
gboolean call_func(GHashTable *hash,
		   const gchar *lib,const gchar *func,
		   const gint argc, const gchar *argv[]);

#endif /* QUEST_MODULES_H */
