#ifndef QUEST_NET_H
#define QUEST_NET_H

#include <gnome.h>
#include <gnet/inetaddr.h>
#include <gnet/gnet.h>
#include <modules.h> /* parse_buffer needs call_func() */


#define QUEST_PORT 4242

/* This is a sly trick to get to the loaded_modules GHashTable
 * when you don't know if handling_iofunc() is called by a Client or
 * a Server...
 */
typedef struct _ClientOrServer {

  GHashTable *loaded_modules;
  GHashTable *users;

  /* the other fields of Clients and Servers differ... */

} ClientOrServer;

extern ClientOrServer *client_or_server;

/*
 * CONNECTION FUNCTIONS
 */

#define SOCKET_BUFFER_SIZE (1024)
#define ANY_IO_CONDITION (G_IO_IN|G_IO_OUT|G_IO_PRI|G_IO_ERR|G_IO_HUP|G_IO_NVAL)

typedef struct _Connection {

  GTcpSocket *socket;
  GIOChannel *iochannel;
  gchar *ip;
  gint port;
  gchar buffer[SOCKET_BUFFER_SIZE];
  guint n;
  gint watch_tag;

} Connection;

/* These functions create a clean new Connection structure
 * and destroy is after use... they are used by accepting_iofunc()
 * and handling_iofunc() when getting new connections and when
 * old connections hang up.
 */
Connection *connection_new();
void connection_destroy(Connection *c);

/* You can use these functions to open a connection to another
 * host, for example the server... you need to initialize a connection
 * because you can open it.
 */
gboolean connection_open(Connection *c);
void connection_close(Connection *c);

/* this function sets a Connection struct's socket and iochannel
 * up as a listener, accepting incoming connections...
 * accepting_iofunc() accepts the incoming request and assigns
 * handling_iofunc() to handle it whenever the iochannel raises
 * an event, e.g. G_IO_IN or G_IO_HUP...
 */
void connection_listen(Connection *c,gint port);
gboolean accepting_iofunc(GIOChannel* iochannel,GIOCondition condition, 
			  gpointer data);
gboolean handling_iofunc(GIOChannel* iochannel,GIOCondition condition,
			 gpointer data);


/*
 * NETWORK STRING AND ARGUMENT FUNCTIONS
 */

#define SEPARATOR ("*") /* ("\027") */
#define SEPARATOR_STRLEN (1)

/* !!! THIS FUNCTION IS THE QUEST PROTOCOL'S MOST IMPORTANT PART !!!
 * this function creates a newly allocated string from all its parameters
 */
gchar *assemble_network_string(const gchar *lib,const gchar *func,
			       gint argc,const gchar *argv[]);

gboolean parse_buffer(Connection *c,ClientOrServer *cos);

/* you can use this function to call a function on a remote host
 * in a nice way... it triggers the iofunc functions on the other side
 * which in turn trigger parse_buffer() on the other side...
 */
gboolean call_remote_func(Connection *c,
			  const gchar *lib,const gchar *func,
			  gchar *argv[]);

/* an auxiliar function used by call_remote_func() */
gint get_argc_from_argv(gchar *argv[]);


#endif /* QUEST_NET_H */
