#include <user.h>

gchar *user_state_string[] = {

  "USER_OFFLINE",
  "USER_ONLINE",
  "USER_AWAY",
  "USER_INVISIBLE",
  NULL

};
    
gchar *user_state_string_nice[] = {

  "offline",
  "online",
  "away",
  "invisible",
  NULL
  
};

guint user_hash_func(gconstpointer key) {

  return g_str_hash(key);

}

gint user_compare_func(gconstpointer a,gconstpointer b) {

  return (strcmp((gchar*)a,(gchar*)b)==0);

}

gboolean user_remove_func(gpointer key,gpointer value,gpointer user_data) {

  user_destroy((User*)value);

  /* true in this case means remove the user from the hash,
   * no questions asked...
   */
  return TRUE;
  
}

User *user_new() {

  User *new = (User*)g_malloc(sizeof(User));

  new->name = NULL;
  new->password = NULL;
  new->user_state = USER_OFFLINE;
  new->c = NULL;

  return new;

}

void user_destroy(User *user) {

  if(user->name != NULL) {
    g_free(user->name);
  }
  if(user->password != NULL) {
    g_free(user->password);
  }

  /* you don't have to free the connection,
   * it gets destroyed when the client hangs up...
   *
   * FIXME: this is a problem when the server program terminates
   * it will be a memory leak.
   */

  g_free(user);

}

User *user_new_with_values(gchar *name,gchar *password,Connection *c) {

  User *new = user_new();

  new->name = name;
  new->password = password;
  new->c = c;

  return new;

}

gboolean user_add(GHashTable *users,User *user) {

  gboolean rv = FALSE;

  g_print("looking up %s %s\n",user->name,user->password);

  if(g_hash_table_lookup(users,user->name)!=NULL) {

    /* user already exists */

  } else {
    User *tmp;

    g_hash_table_insert(users,user->name,user);
    rv = TRUE;

    tmp = user_get(users,user->name);

    g_print("%s has been added\n",tmp->name);

  }

  return rv;

}

gboolean user_remove(GHashTable *users,gchar *name) {

  gboolean rv = FALSE;
  User *user = NULL;

  if((user=g_hash_table_lookup(users,name))!=NULL) {

    g_hash_table_remove(users,user->name);
    user_destroy(user);
    rv = TRUE;

  } else {

    /* user doesn't exist */

  }

  return rv;

}

User *user_get(GHashTable *users,gchar *name) {

  return g_hash_table_lookup(users,name);

}
