#include <server.h>

Server *server_new(gint port) {

  Server *new = (Server*)g_malloc(sizeof(Server));

  new->loaded_modules = module_hash_new();

  new->listener = connection_new();
  connection_listen(new->listener,port);

  new->users = g_hash_table_new(user_hash_func,
				user_compare_func);

  /* GUI */

  new->app = NULL;
  new->ctree = NULL;

  private_plugin_dir = NULL;
  client_or_server = (ClientOrServer*)new;

  return new;

}

void server_destroy_cb(GtkWidget *widget,gpointer data) {

  server_destroy((Server*)data);

}

void server_destroy(Server *s) {

  gtk_widget_destroy(s->app);
  module_hash_destroy(s->loaded_modules);

  connection_destroy(s->listener);

  g_hash_table_foreach_remove(s->users,user_remove_func,NULL);

  g_free(s);

  gtk_main_quit();

}

void server_init(Server *s,gchar *geometry) {
  
  gchar *str;
  
  g_assert(s != NULL);

  /* GUI */

  s->app = 
    gnome_app_new(PACKAGE,(str=g_strdup_printf(_("%s %s"),PACKAGE,VERSION)));

   if(geometry != NULL) {
    gint x,y,width,height;
    if(gnome_parse_geometry(geometry,&x,&y,&width,&height)) {
      if(x != -1) {
	gtk_widget_set_uposition(s->app,x,y);
      }
      if(width != -1) {
	gtk_window_set_default_size(GTK_WINDOW(s->app),width,height);
      }
    } else {
      g_error(_("Could not parse geometry string `%s'"),geometry);
    }
  }
 
  gtk_signal_connect(GTK_OBJECT(s->app),"destroy",
		     GTK_SIGNAL_FUNC(server_destroy_cb),s);

  s->ctree = gtk_ctree_new(3,0);

  gnome_app_set_contents(GNOME_APP(s->app),s->ctree);

  gtk_widget_show_all(s->app);
  
  g_free(str);
  
}
